///////////////////////////////////////////////////
// AGXLED.cpp
// 720020-1172x
//
// Applied Data Systems
//
// See AGXLED.h for a description and history
// 
///////////////////////////////////////////////////
#include "agxled.h"

#define	MEMORY_SIZE					0x10000  // 64kiB memory space; max allocation for VirtualAlloc is 64MiB
                                     // must be 64kiB aligned (VirtualAlloc() requirement)
#define AGX_PXA_GPIO_BASE_ADDR		0x40E00000
#define AGX_PXA_GPLR0_OFFSET		0x00	// level
#define AGX_PXA_GPDR0_OFFSET		0x0C	// direction
#define AGX_PXA_GPSR0_OFFSET		0x18	// set
#define AGX_PXA_GPCR0_OFFSET		0x24	// clear
#define AGX_LED_BIT_SHIFT			0x13

// VirtualCopy definition
extern "C" BOOL VirtualCopy(LPVOID lpvDestMem, LPVOID lpvSrcMem, 
							DWORD dwSizeInBytes, DWORD dwProtectFlag);

LPVOID MapAddress(DWORD baseAddress);
void UnmapAddress(LPVOID address);

AGXLED::AGXLED()
{
	// initialize the direction bits as output
	g_hLED = (DWORD*)MapAddress(AGX_PXA_GPIO_BASE_ADDR);

	// read current direction
	DWORD direction = g_hLED[AGX_PXA_GPDR0_OFFSET / sizeof(DWORD)];
	DEBUGMSG(1, (_T("Current direction register value: 0x%08x\r\n"), direction));

	// set the LED GPIOs as outputs
	direction |= (0x7 << AGX_LED_BIT_SHIFT);
	DEBUGMSG(1, (_T("Setting direction register to: 0x%08x..."), direction));
	g_hLED[AGX_PXA_GPDR0_OFFSET / sizeof(DWORD)] = (direction);
	DEBUGMSG(1, (_T("ok\r\n")));

	// start all LEDs OFF
	WriteLED(0x00);
}

AGXLED::~AGXLED()
{
	UnmapAddress(g_hLED);
}

BYTE AGXLED::ReadLED()
{
	// get the entire level register value
	DWORD state = g_hLED[AGX_PXA_GPLR0_OFFSET / sizeof(DWORD)];

	// shit the LED bits to the LSB
	state = (state >> AGX_LED_BIT_SHIFT);

	// mask off everything but the LED bits
	state &= 0x07;

	return (BYTE)state;
}

void AGXLED::WriteLED(BYTE state)
{
	DWORD set	= 0;
	DWORD clear	= 0;

	// the LED has only 3 bits, so mask off everything else
	DEBUGMSG(1, (_T("state = %i\r\n"), state));
	state &= 0x7;
	DEBUGMSG(1, (_T(" - masked to %i\r\n"), state));

	// the AGX LEDs are not mapped inuitively, so we must reorder
	// the state for display.  The mapping is as follows:
	//
	// data bit   2   1   0
	// part       D9  D8  D7    
	// GPI        20  19  21    <-- note the swap of LED2 and LED1
    
	if(state & 1)
		set |= 4;
	else
		clear |= 4;

	if(state & 2)
		set |= 1;
	else
		clear |= 1;

	if(state & 4)
		set |= 2;
	else
		clear |= 2;

	DEBUGMSG(1, (_T("set = %i : clear = %i\r\n"), set, clear));

	// shift to the LED bits
	set = set << AGX_LED_BIT_SHIFT;
	clear = clear << AGX_LED_BIT_SHIFT;

	// set the high bits
	DEBUGMSG(1, (_T("Setting: 0x%08x..."), set));
	g_hLED[AGX_PXA_GPSR0_OFFSET / sizeof(DWORD)] = set;
	DEBUGMSG(1, (_T("ok\r\n")));

	// clear the low bits
	DEBUGMSG(1, (_T("Clearing: 0x%08x..."), clear));
	g_hLED[AGX_PXA_GPCR0_OFFSET / sizeof(DWORD)] = clear;
	DEBUGMSG(1, (_T("ok\r\n")));
}


/////////////////////////////////////////////
// MapAddress 
//  Maps the desired memory address.
//  This function should be called first and its
//  return value is then passed to the ReadData16 or
//  WriteData16 functions as the baseAddress.
//
// [I] baseAddress - base address to map
// [O] Virtual Address pointer to mapped region
// 
/////////////////////////////////////////////
LPVOID MapAddress(DWORD baseAddress)
{
	LPVOID pAddr = VirtualAlloc(0, MEMORY_SIZE, MEM_RESERVE, PAGE_NOACCESS);
	
	if(pAddr)	
	{
		if(VirtualCopy (pAddr, (PVOID)(baseAddress >> 8), 
				MEMORY_SIZE, PAGE_PHYSICAL|PAGE_READWRITE|PAGE_NOCACHE) == TRUE) 
		{
				DEBUGMSG(1, (_T("0x%08x mapped to: 0x%08x\r\n"), baseAddress, pAddr));
		}
		else 
		{
			RETAILMSG(1, (_T("Error! at VirtualCopy()\r\n")));

			return NULL;
		}
	}
	else 
	{
		RETAILMSG(1, (_T("Error! at VirtualAlloc()\r\n")));
		return NULL;
	}

	return pAddr;
}

void UnmapAddress(LPVOID address)
{
	VirtualFree(address, 0, MEM_RELEASE);
	address = NULL;
}
